import yaml
from telethon import events

from tgcf import config

from .utils import admin_protect, display_forwards, get_args, remove_source


@admin_protect
async def forward_command_handler(event):
    notes = """The `/forward` command allows you to add a new forward.
    Example: suppose you want to forward from a to (b and c)

    ```
    /forward source: a
    dest: [b,c]
    ```

    a,b,c are chat ids

    """.replace(
        "    ", ""
    )

    try:
        args = get_args(event.message.text)
        if not args:
            raise ValueError(f"{notes}\n{display_forwards(config.CONFIG.forwards)}")

        parsed_args = yaml.safe_load(args)
        print(parsed_args)
        forward = config.Forward(**parsed_args)
        print(forward)
        config.CONFIG.forwards.append(forward)
        config.from_to = config.load_from_to(config.CONFIG.forwards)

        await event.respond("Success")
        config.update_config_file(config.CONFIG)
    except ValueError as err:
        print(err)
        await event.respond(str(err))

    finally:
        raise events.StopPropagation


@admin_protect
async def remove_command_handler(event):
    notes = """The `/remove` command allows you to remove a source from forwarding.
    Example: Suppose you want to remove the channel with id -100, then run

    `/remove source: -100`

    """.replace(
        "    ", ""
    )

    try:
        args = get_args(event.message.text)
        if not args:
            raise ValueError(f"{notes}\n{display_forwards(config.CONFIG.forwards)}")

        parsed_args = yaml.safe_load(args)
        print(parsed_args)
        source_to_remove = parsed_args.get("source")
        print(source_to_remove)
        config.CONFIG.forwards = remove_source(source_to_remove, config.CONFIG.forwards)
        print(config.CONFIG.forwards)
        config.from_to = config.load_from_to(config.CONFIG.forwards)

        await event.respond("Success")
        config.update_config_file(config.CONFIG)
    except ValueError as err:
        print(err)
        await event.respond(str(err))

    finally:
        raise events.StopPropagation


async def start_command_handler(event):
    await event.respond("I am alive!")


async def help_command_handler(event):
    await event.respond(
        "Please read the documentation at https://github.com/aahnik/tgcf"
    )



import logging
from importlib import import_module
from typing import List

from tgcf.config import CONFIG

PLUGINS = CONFIG.plugins

plugins = []


def load_plugins():
    plugins = []
    for plugin_id, plugin_data in PLUGINS.items():
        plugin_module_name = f"tgcf_{plugin_id}"
        plugin_class_name = f"Tgcf{plugin_id.title()}"
        try:
            plugin_module = import_module(plugin_module_name)
            PluginClass = getattr(plugin_module, plugin_class_name)
            plugin = PluginClass(plugin_data)
            assert plugin.id == plugin_id

        except ModuleNotFoundError:
            logging.error(f"Could not find plugin for {plugin_id}")
        except AttributeError:
            logging.error(f"Found plugin {plugin_id}, but failed to load.")
        else:
            print(f"Loaded plugin {plugin_id}")
            plugins.append(plugin)
    return plugins


plugins = load_plugins()


def apply_plugins(message) -> List:
    for plugin in plugins:
        message = plugin.modify(message)
        logging.info(f"Applied plugin {plugin.id}")
        if not message:
            return
    return message



import logging
from enum import Enum
from typing import List, Optional

from pydantic import BaseModel


class FilterList(BaseModel):
    blacklist: Optional[List[str]] = []
    whitelist: Optional[List[str]] = []


class FileType(str, Enum):
    AUDIO = "audio"
    GIF = "gif"
    VIDEO = "video"
    VIDEO_NOTE = "video_note"
    STICKER = "sticker"
    CONTACT = "contact"
    PHOTO = "photo"
    DOCUMENT = "document"
    NOFILE = "nofile"


class FilesFilterList(BaseModel):
    blacklist: Optional[List[FileType]] = []
    whitelist: Optional[List[FileType]] = []


class TextFilter(FilterList):
    case_sensitive: Optional[bool] = False


class Filters(BaseModel):
    users: Optional[FilterList] = FilterList()
    files: Optional[FilesFilterList] = FilesFilterList()
    text: Optional[TextFilter] = TextFilter()


class TgcfFilter:
    id = "filter"

    def __init__(self, data):
        print("tgcf filter data loaded")
        self.filters = Filters(**data)
        self.case_correct()
        logging.info(self.filters)

    def case_correct(self):
        textf: TextFilter = self.filters.text

        if textf.case_sensitive is False:
            textf.blacklist = [item.lower() for item in textf.blacklist]
            textf.whitelist = [item.lower() for item in textf.whitelist]

    def modify(self, message):

        if self.users_safe(message):
            if self.files_safe(message):
                if self.text_safe(message):
                    return message

    def text_safe(self, message):
        flist = self.filters.text

        text = message.text
        if not flist.case_sensitive:
            text = text.lower()
        if not text and flist.whitelist == []:
            return True

        for forbidden in flist.blacklist:
            if forbidden in text:
                return False
        if not flist.whitelist:
            return True
        for allowed in flist.whitelist:
            if allowed in text:
                return True

    def users_safe(self, message):
        flist = self.filters.users
        sender = str(message.sender_id)
        logging.info(f"M message from sender id {sender}")
        if sender in flist.blacklist:
            return False
        if not flist.whitelist:
            return True
        if sender in flist.whitelist:
            return True

    def files_safe(self, message):
        flist = self.filters.files

        def file_type(message):
            file_types = [
                FileType.AUDIO,
                FileType.GIF,
                FileType.VIDEO,
                FileType.VIDEO_NOTE,
                FileType.STICKER,
                FileType.CONTACT,
                FileType.PHOTO,
                FileType.DOCUMENT,
            ]
            for _type in file_types:
                obj = getattr(message, _type)
                if obj:
                    return _type

        fl_type = file_type(message)
        if not fl_type:
            fl_type = FileType.NOFILE
        print(fl_type)
        if fl_type in flist.blacklist:
            return False
        if not flist.whitelist:
            return True
        if fl_type in flist.whitelist:
            return True



""" This module implements the command line interface for tgcf,
using the modern and robust `typer`.
"""

import asyncio
import logging
import os
from enum import Enum
from typing import Optional

import typer
from dotenv import load_dotenv

from tgcf import __version__

load_dotenv(".env")

FAKE = bool(os.getenv("FAKE"))
app = typer.Typer(add_completion=False)


class Mode(str, Enum):
    """tgcf works in two modes"""

    past = "past"
    live = "live"


def version_callback(value: bool):
    if value:
        print(__version__)
        raise typer.Exit()


def verbosity_callback(value: bool):
    if value:
        level = logging.INFO
    else:
        level = logging.WARNING
    logging.basicConfig(level=level)
    logging.info("Verbosity turned on. \nThis is suitable for debugging.\n")


@app.command()
def main(
    mode: Mode = typer.Argument(
        ..., help="Choose the mode in which you want to run tgcf.", envvar="TGCF_MODE"
    ),
    verbose: Optional[bool] = typer.Option(
        None,
        "--loud",
        "-l",
        callback=verbosity_callback,
        envvar="LOUD",
        help="Increase output verbosity.",
    ),
    version: Optional[bool] = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        help="Show version and exit.",
    ),
):
    """tgcf is a powerful tool for forwarding telegram messages from source to destination.

    Don't forget to star  https://github.com/aahnik/tgcf

    Telegram Channel https://telegram.me/tg_cf
    """

    if FAKE:
        print(f"mode = {mode}")
        quit(1)

    if mode == mode.past:
        from tgcf.past import forward_job

        asyncio.run(forward_job())
    else:
        from tgcf.live import start_sync

        start_sync()


# AAHNIK 2021



from typing import List

from telethon import events

from tgcf import config
from tgcf.config import Forward


def admin_protect(org_func):
    if config.CONFIG.admins == []:
        return org_func

    async def wrapper_func(event):
        if event.sender_id not in config.CONFIG.admins:
            await event.respond("You are not authorized.")
            raise events.StopPropagation
        return await org_func(event)

    return wrapper_func


def get_args(text: str):
    splitted = text.split(" ", 1)
    if not len(splitted) == 2:
        return ""
    else:
        prefix, args = splitted
    print(prefix)
    args = args.strip()
    print(args)
    return args


def display_forwards(forwards: List[Forward]):
    if len(forwards) == 0:
        return "Currently no forwards are set"
    forward_str = "This is your configuration"
    for forward in forwards:
        forward_str = (
            forward_str
            + f"\n\n```\nsource: {forward.source}\ndest: {forward.dest}\n```\n"
        )

    return forward_str


def remove_source(source, forwards: List[Forward]):
    for i, forward in enumerate(forwards):
        print(forward)
        print(type(forward.source))
        print(type(source))
        if forward.source == source:

            del forwards[i]

            return forwards

    else:
        raise ValueError("The source does not exist")



from tgcf.config import CONFIG


async def send_message(client, *args):
    # show forwarded from
    if CONFIG.show_forwarded_from:
        return await client.forward_messages(*args)
    return await client.send_message(*args)
